@@ -75,12 +75,12 @@ typedef enum
    } WIZSTEP;
 
 static WIZARD_STATE aStates[] = {
-   { STEP_START,     IDD_WIZ_START,     WizStart_DlgProc     },
-   { STEP_STARTING,  IDD_WIZ_STARTING,  WizStarting_DlgProc  },
-   { STEP_CREDS,     IDD_WIZ_CREDS,     WizCreds_DlgProc     },
-   { STEP_MOUNT,     IDD_WIZ_MOUNT,     WizMount_DlgProc     },
-   { STEP_MOUNTING,  IDD_WIZ_MOUNTING,  WizMounting_DlgProc  },
-   { STEP_FINISH,    IDD_WIZ_FINISH,    WizFinish_DlgProc    },
+   { STEP_START,     IDD_WIZ_START,     (DLGPROC)WizStart_DlgProc,	0 },
+   { STEP_STARTING,  IDD_WIZ_STARTING,  (DLGPROC)WizStarting_DlgProc,	0 },
+   { STEP_CREDS,     IDD_WIZ_CREDS,     (DLGPROC)WizCreds_DlgProc,	0 },
+   { STEP_MOUNT,     IDD_WIZ_MOUNT,     (DLGPROC)WizMount_DlgProc,	0 },
+   { STEP_MOUNTING,  IDD_WIZ_MOUNTING,  (DLGPROC)WizMounting_DlgProc,	0 },
+   { STEP_FINISH,    IDD_WIZ_FINISH,    (DLGPROC)WizFinish_DlgProc,	0 }
 };
 
 static const int cStates = sizeof(aStates) / sizeof(aStates[0]);
@@ -231,7 +231,7 @@ void WizStarting_OnInitDialog (HWND hDlg)
                                        SERVICE_QUERY_STATUS | SERVICE_START)) != NULL)
          {
          if (StartService (hService, 0, 0))
-			TestAndDoMapShare(SERVICE_START_PENDING);
+             TestAndDoMapShare(SERVICE_START_PENDING);
 
          CloseServiceHandle (hService);
          }
@@ -313,7 +313,7 @@ BOOL CALLBACK WizCreds_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
 
                      WizCreds_OnEnable (hDlg, FALSE);
 
-                     if (ObtainNewCredentials (szCell, szUser, szPassword) == 0)
+                     if (ObtainNewCredentials (szCell, szUser, szPassword, FALSE) == 0)
                         {
                         g.pWizard->SetState (STEP_MOUNT);
                         }
@@ -345,7 +345,6 @@ void WizCreds_OnInitDialog (HWND hDlg)
    HKEY hk;
 
    TCHAR szCell[ cchRESOURCE ] = TEXT("");
-   (void)GetDefaultCell (szCell);
    SetDlgItemText (hDlg, IDC_NEWCREDS_CELL, szCell);
 
    TCHAR szUser[ cchRESOURCE ] = TEXT("");
