@@ -28,7 +28,9 @@ struct nfsclientpag {
     afs_int32 uid;		/* search based on uid and ... */
     afs_int32 host;		/* ... nfs client's host ip address */
     afs_int32 pag;		/* active pag for all  (uid, host) "unpaged" conns */
-    char *sysname;		/* user's "@sys" value; also kept in unixuser */
+    afs_int32 client_uid;       /* actual UID on client */
+    char *sysname[MAXNUMSYSNAMES];/* user's "@sys" value; also kept in unixuser */
+    int sysnamecount;           /*  number of sysnames */
     afs_int32 lastcall;		/*  Used for timing out nfsclientpag structs  */
 };
 
