@@ -8,18 +8,15 @@
  */
 
 #include <afsconfig.h>
-#include <afs/param.h>
+#include <afs/stds.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bucoord/bc_status.c,v 1.10 2003/11/23 04:53:30 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/bucoord/bc_status.c,v 1.11.14.2 2007/04/10 18:39:50 shadow Exp $");
 
-#include <afs/stds.h>
 #include <sys/types.h>
-#include <stdio.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #else
-#include <strings.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -41,6 +38,10 @@ RCSID
     curPollPtr->flags &= ~(clear);		\
     unlock_Status();
 
+extern struct bc_config *bc_globalConfig;
+extern afs_int32 bc_GetConn(struct bc_config *aconfig, afs_int32 aport, struct rx_connection **tconn);
+extern statusP findStatus(afs_uint32 taskId);
+
 /* globals for backup coordinator status management */
 
 dlqlinkT statusHead;		/* chain of status blocks */
@@ -117,7 +118,7 @@ cmdDispatch()
 
     code = cmd_ParseLine(internalCmdLine, targv, &targc, MAXV);
     if (code) {
-	printf("Couldn't parse line: '%s'", error_message(code));
+	printf("Couldn't parse line: '%s'", afs_error_message(code));
 	return (1);
     }
     free(internalCmdLine);
@@ -149,8 +150,6 @@ statusWatcher()
     PROCESS dispatchPid;
 
     afs_int32 code = 0;
-    extern struct bc_config *bc_globalConfig;
-    extern struct rx_connection *bc_GetConn();
 
     lastTaskCode = 0;
 
@@ -267,8 +266,8 @@ statusWatcher()
 	if (localTaskFlags & ABORT_REQUEST) {
 	    code = TC_RequestAbort(tconn, taskId);
 	    if (code) {
-		com_err("statusWatcher", code, "; Can't post abort request");
-		com_err("statusWatcher", 0, "...Deleting job");
+		afs_com_err("statusWatcher", code, "; Can't post abort request");
+		afs_com_err("statusWatcher", 0, "...Deleting job");
 		if (localTaskFlags & NOREMOVE) {
 		    curPollPtr->flags |= (STARTING | TASK_ERROR);
 		    curPollPtr->scheduledDump = 0;
@@ -431,7 +430,6 @@ waitForTask(taskId)
 {
     statusP ptr;
     afs_int32 done = 0, rcode, t;
-    extern statusP findStatus();
 
     t = (TASK_DONE | ABORT_DONE | TASK_ERROR);
     while (!done) {
