@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/kaserver.c,v 1.19.2.2 2006/06/20 20:37:11 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/kauth/kaserver.c,v 1.19.2.4 2007/04/10 18:39:52 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -47,6 +47,7 @@ RCSID
 #include <afs/cellconfig.h>
 #include <lock.h>
 #include <afs/afsutil.h>
+#include <afs/com_err.h>
 #include <ubik.h>
 #include <sys/stat.h>
 #include "kauth.h"
@@ -330,7 +331,7 @@ main(argc, argv)
     if (!KA_conf) {
 	code = KANOCELLS;
       abort:
-	com_err(whoami, code, "Failed getting cell info");
+	afs_com_err(whoami, code, "Failed getting cell info");
 	exit(1);
     }
 #ifdef        AUTH_DBM_LOG
@@ -343,12 +344,20 @@ main(argc, argv)
     OpenLog(AFSDIR_SERVER_KALOG_FILEPATH);
     SetupLogSignals();
 #endif
+
+    fprintf(stderr, "%s: WARNING: kaserver is deprecated due to its weak security "
+	    "properties.  Migrating to a Kerberos 5 KDC is advised.  "
+	    "http://www.openafs.org/no-more-des.html\n", whoami);
+    ViceLog(0, ("WARNING: kaserver is deprecated due to its weak security properties.  "
+	    "Migrating to a Kerberos 5 KDC is advised.  "
+	    "http://www.openafs.org/no-more-des.html\n"));
+
     code =
 	afsconf_GetExtendedCellInfo(KA_conf, cell, AFSCONF_KAUTHSERVICE,
 				    &cellinfo, &clones);
     if (servers) {
 	if (code = ubik_ParseServerList(argc, argv, &myHost, serverList)) {
-	    com_err(whoami, code, "Couldn't parse server list");
+	    afs_com_err(whoami, code, "Couldn't parse server list");
 	    exit(1);
 	}
 	cellinfo.hostAddr[0].sin_addr.s_addr = myHost;
@@ -416,7 +425,7 @@ main(argc, argv)
 				  &clones, dbpath, &KA_dbase);
 
     if (code) {
-	com_err(whoami, code, "Ubik init failed");
+	afs_com_err(whoami, code, "Ubik init failed");
 	exit(2);
     }
 
