@@ -17,7 +17,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.27.2.3 2005/01/31 04:20:00 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.34.2.1 2007/06/06 17:22:05 shadow Exp $");
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -33,6 +33,7 @@ RCSID("$Header: /cvs/openafs/src/lwp/lwp.c,v 1.27.2.3 2005/01/31 04:20:00 shadow
 #include <sys/pseg.h>
 #include <sys/core.h>
 #pragma alloca
+int setlim(int limcon, uchar_t hard, int limit);
 #endif
 #ifdef AFS_SGI64_ENV
 extern char *getenv();
@@ -80,6 +81,8 @@ extern char PRE_Block;		/* from preempt.c */
 #ifdef __s390__
 #define MINFRAME    96
 #define STACK_ALIGN 8
+#elif defined(AFS_DARWIN_ENV)
+#define STACK_ALIGN 16
 #else
 #define STACK_ALIGN 4
 #endif
@@ -145,9 +148,7 @@ int lwp_nextindex;
 int lwp_MinStackSize = 0;
 
 static int
-lwp_remove(p, q)
-     register PROCESS p;
-     register struct QUEUE *q;
+lwp_remove(register PROCESS p, register struct QUEUE *q)
 {
     /* Special test for only element on queue */
     if (q->count == 1)
@@ -166,9 +167,7 @@ lwp_remove(p, q)
 }
 
 static int
-insert(p, q)
-     register PROCESS p;
-     register struct QUEUE *q;
+insert(register PROCESS p, register struct QUEUE *q)
 {
     if (q->head == NULL) {	/* Queue is empty */
 	q->head = p;
@@ -184,9 +183,7 @@ insert(p, q)
 }
 
 static int
-move(p, from, to)
-     PROCESS p;
-     struct QUEUE *from, *to;
+move(PROCESS p, struct QUEUE *from, struct QUEUE *to)
 {
 
     lwp_remove(p, from);
@@ -248,8 +245,7 @@ LWP_QWait(void)
 }
 
 int
-LWP_QSignal(pid)
-     register PROCESS pid;
+LWP_QSignal(register PROCESS pid)
 {
     if (pid->status == QWAITING) {
 	pid->status = READY;
@@ -279,6 +275,7 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 #else
     char *stackptr;
 #endif
+    char *stackmemory;
 
 #if defined(AFS_LWP_MINSTACKSIZE)
     /*
@@ -303,7 +300,11 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 	    return LWP_ENOMEM;
 	}
 	if (stacksize < MINSTACK)
+#ifdef AFS_DARWIN_ENV
+	    stacksize = 1008;
+#else /* !AFS_DARWIN_ENV */
 	    stacksize = 1000;
+#endif /* !AFS_DARWIN_ENV */
 	else
 	    stacksize =
 		STACK_ALIGN * ((stacksize + STACK_ALIGN - 1) / STACK_ALIGN);
@@ -346,20 +347,30 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 	    stackptr = reserveFromStack(lwp_MaxStackSize);
 	}
 	stackptr -= stacksize;
+	stackmemory = stackptr;
 #else
-	if ((stackptr = (char *)malloc(stacksize + 7)) == NULL) {
+#ifdef AFS_DARWIN_ENV
+	if ((stackmemory = (char *)malloc(stacksize + STACK_ALIGN - 1)) == NULL)
+#else /* !AFS_DARWIN_ENV */
+	if ((stackmemory = (char *)malloc(stacksize + 7)) == NULL)
+#endif /* !AFS_DARWIN_ENV */
+	{
 	    Set_LWP_RC();
 	    return LWP_ENOMEM;
 	}
 	/* Round stack pointer to byte boundary */
-	stackptr = (char *)(8 * (((long)stackptr + 7) / 8));
+#ifdef AFS_DARWIN_ENV
+	stackptr = (char *)(STACK_ALIGN * (((long)stackmemory + STACK_ALIGN - 1) / STACK_ALIGN));
+#else /* !AFS_DARWIN_ENV */
+	stackptr = (char *)(8 * (((long)stackmemory + 7) / 8));
+#endif /* !AFS_DARWIN_ENV */
 #endif
 	if (priority < 0 || priority >= MAX_PRIORITIES) {
 	    Set_LWP_RC();
 	    return LWP_EBADPRI;
 	}
 	Initialize_Stack(stackptr, stacksize);
-	Initialize_PCB(temp, priority, stackptr, stacksize, ep, parm, name);
+	Initialize_PCB(temp, priority, stackmemory, stacksize, ep, parm, name);
 	insert(temp, &runnable[priority]);
 	temp2 = lwp_cpptr;
 	if (PRE_Block != 0)
@@ -377,8 +388,12 @@ LWP_CreateProcess(int (*ep) (), int stacksize, int priority, void *parm,
 		    stackptr + MINFRAME);
 #else
 #if defined(AFS_SGI62_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+#ifdef sys_x86_darwin_80
+	savecontext(Create_Process_Part2, &temp2->context, stackptr + stacksize - 16 - sizeof(void *));	/* 16 = 2 * jmp_buf_type */
+#else /* !sys_x86_darwin_80 */
 	/* Need to have the sp on an 8-byte boundary for storing doubles. */
 	savecontext(Create_Process_Part2, &temp2->context, stackptr + stacksize - 16);	/* 16 = 2 * jmp_buf_type */
+#endif /* !sys_x86_darwin_80 */
 #else
 #if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_SPARC_LINUX20_ENV)
 	savecontext(Create_Process_Part2, &temp2->context, stackptr + stacksize - 0x40);	/* lomgjmp does something
@@ -835,7 +850,6 @@ Dump_One_Process(PROCESS pid)
 	       pid->stacksize, pid->stack);
 	printf("***LWP: HWM stack usage: ");
 	printf("%d\n", Stack_Used(pid->stack, pid->stacksize));
-	free(pid->stack);
     }
     printf("***LWP: Current Stack Pointer: 0x%x\n", pid->context.topstack);
     if (pid->eventcnt > 0) {
@@ -999,7 +1013,9 @@ Free_PCB(PROCESS pid)
 	Debug(0,
 	      ("HWM stack usage: %d, [PCB at 0x%x]",
 	       Stack_Used(pid->stack, pid->stacksize), pid));
+#ifndef AFS_AIX32_ENV
 	free(pid->stack);
+#endif
     }
     if (pid->eventlist != NULL)
 	free(pid->eventlist);
