@@ -2,7 +2,7 @@
  *	bit map routines (in-core).
  */
 /*
- * Copyright (c) 1995, 1996 Marcus D. Watts  All rights reserved.
+ * Copyright (c) 1995, 1996, 2007 Marcus D. Watts  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -12,10 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by Marcus D. Watts.
- * 4. The name of the developer may not be used to endorse or promote
+ * 3. The name of the developer may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -35,7 +32,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/map.c,v 1.2 2003/07/15 23:16:02 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/map.c,v 1.3.14.1 2007/05/16 19:44:12 shadow Exp $");
 
 #ifdef SUPERGROUPS
 #include <errno.h>
@@ -52,13 +49,13 @@ char *malloc();
 struct bitmap {
     struct bitmap *m_next;
     int m_page;
-    long m_data[MDATA];
+    int m_data[MDATA];
 };
 
-#define MAP(p)	((struct bitmap*)((long)(p)&~1))
+#define MAP(p)	((struct bitmap*)((int)(p)&~1))
 #define NEGMAP(p)	(((int)(p))&1)
 #define POSMAP(p)	(!NEGMAP(p))
-#define NOT_MAP(mp)	((struct map *) (((long)(mp)) ^ 1))
+#define NOT_MAP(mp)	((struct map *) (((int)(mp)) ^ 1))
 
 #define NUMBERTOBIT(n)	((n) & ((1<<LSHIFT)-1))
 #define NUMBERTOINDEX(n)	((n>>LSHIFT) & ((1<<MSHIFT)-1))
@@ -69,18 +66,11 @@ struct bitmap {
 #define Aflag	(debug_mask & (1L<<('Z'-'@')))
 extern int debug_mask;
 
-#if __STDC__
 int
-in_map(struct map *parm, long node)
-#else
-int
-in_map(parm, node)
-     struct map *parm;
-     long node;
-#endif
+in_map(struct map *parm, int node)
 {
     struct bitmap *map;
-    long bit;
+    int bit;
     int x, page;
     int result;
 
@@ -98,7 +88,7 @@ in_map(parm, node)
     if (Aflag)
 	if (TONUMBER(page, x, bit) != node) {
 	    printf
-		("bxp mixup: node=%ld -> p=%d x=%d b=%d -> %ld, %ld, %ld = %ld\n",
+		("bxp mixup: node=%d -> p=%d x=%d b=%d -> %d, %d, %d = %d\n",
 		 node, page, x, bit, TONUMBER(page, 0, 0), TONUMBER(0, x, 0),
 		 TONUMBER(0, 0, bit), TONUMBER(page, x, bit));
 	}
@@ -117,8 +107,7 @@ in_map(parm, node)
 }
 
 void
-free_map(parm)
-     struct map *parm;
+free_map(struct map *parm)
 {
     struct bitmap *map, *next;
 #ifdef MAP_DEBUG
@@ -137,12 +126,10 @@ free_map(parm)
 }
 
 struct map *
-add_map(parm, node)
-     struct map *parm;
-     long node;
+add_map(struct map *parm, int node)
 {
     struct bitmap *map;
-    long bit;
+    int bit;
     int x, page;
 
 #ifdef MAP_DEBUG
@@ -171,7 +158,7 @@ add_map(parm, node)
 	    return 0;
 	}
 	map->m_page = page;
-	bzero((char *)map->m_data, sizeof map->m_data);
+	memset((char *) map->m_data, 0, sizeof map->m_data);
 	if (NEGMAP(parm)) {
 	    int i;
 	    for (i = 0; i < MDATA; ++i)
@@ -198,14 +185,13 @@ add_map(parm, node)
 }
 
 struct bitmap *
-simplify_bitmap(map)
-     struct bitmap *map;
+simplify_bitmap(struct bitmap *map)
 {
     struct bitmap **mpp, *mp2;
     int i;
-    for (mpp = &map; mp2 = *mpp;) {
+    for (mpp = &map; (mp2 = *mpp);) {
 	for (i = 0; i < MDATA; ++i)
-	    if (map->m_data[i])
+	    if (mp2->m_data[i])
 		break;
 	if (i == MDATA) {
 #ifdef PRINT_MAP_ERROR
@@ -221,13 +207,12 @@ simplify_bitmap(map)
 }
 
 struct bitmap *
-or_bitmap(left, right)
-     struct bitmap *left, *right;
+or_bitmap(struct bitmap *left, struct bitmap *right)
 {
     struct bitmap **rightmp, *lmap, *rmap;
     int i;
     for (lmap = left; lmap; lmap = lmap->m_next) {
-	for (rightmp = &right; rmap = *rightmp; rightmp = &rmap->m_next)
+	for (rightmp = &right; (rmap = *rightmp); rightmp = &rmap->m_next)
 	    if (rmap->m_page == lmap->m_page) {
 		for (i = 0; i < MDATA; ++i)
 		    lmap->m_data[i] |= rmap->m_data[i];
@@ -236,21 +221,21 @@ or_bitmap(left, right)
 		break;
 	    }
     }
-    for (rightmp = &left; *rightmp; rightmp = &(*rightmp)->m_next);
+    for (rightmp = &left; *rightmp; rightmp = &(*rightmp)->m_next)
+	;
     *rightmp = right;
     return left;
 }
 
 struct bitmap *
-and_bitmap(left, right)
-     struct bitmap *left, *right;
+and_bitmap(struct bitmap *left, struct bitmap *right)
 {
     struct bitmap **rightmp, *lmap, *rmap, **leftmp;
     int i;
-    long sig;
-    for (leftmp = &left; lmap = *leftmp;) {
+    int sig;
+    for (leftmp = &left; (lmap = *leftmp);) {
 	sig = 0;
-	for (rightmp = &right; rmap = *rightmp; rightmp = &rmap->m_next)
+	for (rightmp = &right; (rmap = *rightmp); rightmp = &rmap->m_next)
 	    if (rmap->m_page == lmap->m_page) {
 		for (i = 0; i < MDATA; ++i)
 		    sig |= (lmap->m_data[i] &= rmap->m_data[i]);
@@ -271,20 +256,19 @@ and_bitmap(left, right)
 
 /* bit set in left, but not in right */
 struct bitmap *
-bic_bitmap(left, right)
-     struct bitmap *left, *right;
+bic_bitmap(struct bitmap *left, struct bitmap *right)
 {
     struct bitmap **rightmp, *lmap, *rmap, **leftmp;
     int i;
-    long sig;
+    int sig;
 #ifdef MAP_DEBUG
     if (Mflag) {
-	printf("bic_bitmap: left=%#lx right=%#lx\n", left, right);
+	printf("bic_bitmap: left=%#lx right=%#lx\n", (long)left, (long)right);
     }
 #endif
-    for (leftmp = &left; lmap = *leftmp;) {
+    for (leftmp = &left; (lmap = *leftmp);) {
 	sig = 0;
-	for (rightmp = &right; rmap = *rightmp; rightmp = &rmap->m_next)
+	for (rightmp = &right; (rmap = *rightmp); rightmp = &rmap->m_next)
 	    if (rmap->m_page == lmap->m_page) {
 		for (i = 0; i < MDATA; ++i)
 		    sig |= (lmap->m_data[i] &= ~rmap->m_data[i]);
@@ -303,15 +287,14 @@ bic_bitmap(left, right)
     left = simplify_bitmap(left);
 #ifdef MAP_DEBUG
     if (Mflag) {
-	printf("bic_bitmap: result=%#lx\n", left);
+	printf("bic_bitmap: result=%#lx\n", (long) left);
     }
 #endif
     return left;
 }
 
 struct map *
-and_map(mp1, mp2)
-     struct map *mp1, *mp2;
+and_map(struct map *mp1, struct map *mp2)
 {
 #ifdef MAP_DEBUG
     if (Mflag) {
@@ -323,11 +306,12 @@ and_map(mp1, mp2)
 #endif
     if (POSMAP(mp1))
 	if (POSMAP(mp2))
-	    mp1 = (struct map *)and_bitmap(mp1, mp2);
+	    mp1 = (struct map *)and_bitmap((struct bitmap *) mp1,
+		(struct bitmap *) mp2);
 	else
-	    mp1 = (struct map *)bic_bitmap(mp1, MAP(mp2));
+	    mp1 = (struct map *)bic_bitmap((struct bitmap *) mp1, MAP(mp2));
     else if (POSMAP(mp2))
-	mp1 = (struct map *)bic_bitmap(mp2, MAP(mp1));
+	mp1 = (struct map *)bic_bitmap((struct bitmap *) mp2, MAP(mp1));
     else
 	mp1 = NOT_MAP(or_bitmap(MAP(mp1), MAP(mp2)));
 #ifdef MAP_DEBUG
@@ -341,8 +325,7 @@ and_map(mp1, mp2)
 }
 
 struct map *
-or_map(mp1, mp2)
-     struct map *mp1, *mp2;
+or_map(struct map *mp1, struct map *mp2)
 {
 #ifdef MAP_DEBUG
     if (Mflag) {
@@ -354,11 +337,12 @@ or_map(mp1, mp2)
 #endif
     if (POSMAP(mp1))
 	if (POSMAP(mp2))
-	    mp1 = (struct map *)or_bitmap(mp1, mp2);
+	    mp1 = (struct map *)or_bitmap((struct bitmap *) mp1,
+		(struct bitmap *) mp2);
 	else
-	    mp1 = NOT_MAP(bic_bitmap(MAP(mp2), mp1));
+	    mp1 = NOT_MAP(bic_bitmap(MAP(mp2), (struct bitmap *) mp1));
     else if (POSMAP(mp2))
-	mp1 = NOT_MAP(bic_bitmap(MAP(mp1), mp2));
+	mp1 = NOT_MAP(bic_bitmap(MAP(mp1), (struct bitmap *) mp2));
     else
 	mp1 = NOT_MAP(and_bitmap(MAP(mp1), MAP(mp2)));
 #ifdef MAP_DEBUG
@@ -372,8 +356,7 @@ or_map(mp1, mp2)
 }
 
 struct map *
-not_map(map)
-     struct map *map;
+not_map(struct map *map)
 {
 #ifdef MAP_DEBUG
     if (Mflag) {
@@ -386,8 +369,7 @@ not_map(map)
 }
 
 struct map *
-copy_map(parm)
-     struct map *parm;
+copy_map(struct map *parm)
 {
     struct bitmap *result, **mpp, *map;
 #ifdef MAP_DEBUG
@@ -398,7 +380,7 @@ copy_map(parm)
     }
 #endif
     map = MAP(parm);
-    for (mpp = &result; *mpp = 0, map; map = map->m_next) {
+    for (mpp = &result; (*mpp = 0), map; map = map->m_next) {
 	*mpp = (struct bitmap *)malloc(sizeof **mpp);
 	if (!*mpp) {
 #ifdef MAP_DEBUG
@@ -418,18 +400,18 @@ copy_map(parm)
 	return (struct map *)result;
 }
 
-long
-count_map(parm)
-     struct map *parm;
+int
+count_map(struct map *parm)
 {
-    long nf;
+    int nf;
     struct bitmap *map;
-    register i, j;
+    int i, j;
 
     nf = 0;
     for (map = MAP(parm); map; map = map->m_next) {
 	for (i = 0; i < MDATA; ++i) {
-	    if (!map->m_data[i]);
+	    if (!map->m_data[i])
+		;
 	    else if (!~map->m_data[i])
 		nf += (1 << LSHIFT);
 	    else
@@ -450,13 +432,11 @@ count_map(parm)
     return nf;
 }
 
-long
-next_map(parm, node)
-     struct map *parm;
-     long node;
+int
+next_map(struct map *parm, int node)
 {
     struct bitmap *map, *lowest;
-    long bit, mask;
+    int bit, mask;
     int x, page;
     int best;
     int i;
@@ -501,7 +481,7 @@ next_map(parm, node)
 		if (Aflag) {
 		    if (bn == (1 << LSHIFT)) {
 			printf
-			    ("next_map: botch; pageno %d index %d data %#lx mask %#lx x,bit %d,%#lx\n",
+			    ("next_map: botch; pageno %d index %d data %#x mask %#x x,bit %d,%#x\n",
 			     map->m_page, i, map->m_data[i], mask, x, bit);
 			continue;
 		    }
@@ -515,7 +495,7 @@ next_map(parm, node)
 #ifdef MAP_DEBUG
     if (Aflag) {
 	printf(" -> %d\n", best);
-	if (best >= 0 && !in_map((struct map *)parm, best)) {
+	if (best >= 0 && !in_map(parm, best)) {
 	    printf("next_map: botch; %d not in map\n", best);
 	    return -1;
 	}
@@ -524,20 +504,17 @@ next_map(parm, node)
     return best;
 }
 
-long
-first_map(parm)
-     struct map *parm;
+int
+first_map(struct map *parm)
 {
     return next_map(parm, -9999);
 }
 
-long
-prev_map(parm, node)
-     struct map *parm;
-     long node;
+int
+prev_map(struct map *parm, int node)
 {
     struct bitmap *map, *lowest;
-    long bit, mask;
+    int bit, mask;
     int x, page;
     int best;
     int i;
@@ -558,7 +535,7 @@ prev_map(parm, node)
     }
 
     if (node < 0)
-	node = ((unsigned long)~0) >> 1;
+	node = ((unsigned int)~0) >> 1;
 
     --node;
     bit = NUMBERTOBIT(node);
@@ -585,7 +562,7 @@ prev_map(parm, node)
 		if (Aflag) {
 		    if (bn < 0) {
 			printf
-			    ("prev_map: botch; pageno %d index %d data %#lx mask %#lx x,bit %d,%#lx\n",
+			    ("prev_map: botch; pageno %d index %d data %#x mask %#x x,bit %d,%#x\n",
 			     map->m_page, i, map->m_data[i], mask, x, bit);
 			continue;
 		    }
@@ -608,23 +585,20 @@ prev_map(parm, node)
     return best;
 }
 
-long
-last_map(parm)
-     struct map *parm;
+int
+last_map(struct map *parm)
 {
     return prev_map(parm, 0x7fffffff);
 }
 
 struct map *
-negative_map(map)
-     struct map *map;
+negative_map(struct map *map)
 {
     return (struct map *)NEGMAP(map);
 }
 
 struct map *
-bic_map(mp1, mp2)
-     struct map *mp1, *mp2;
+bic_map(struct map *mp1, struct map *mp2)
 {
 #ifdef MAP_DEBUG
     if (Mflag) {
@@ -646,8 +620,8 @@ bic_map(mp1, mp2)
 }
 
 #ifdef PRINT_MAP_ERROR
-print_map(parm)
-     struct map *parm;
+void 
+print_map(struct map *parm)
 {
     struct bitmap *map;
     int i, j;
@@ -657,9 +631,9 @@ print_map(parm)
     }
     map = MAP(parm);
     if (!map)
-	printf(" nil(%lx)", parm);
+	printf(" nil(%lx)", (long)parm);
     else
-	printf(" %lx", parm);
+	printf(" %lx", (long)parm);
     lastbitno = -100;
     firstbitno = -100;
     for (; map; map = map->m_next)
@@ -694,14 +668,11 @@ print_map(parm)
 
 #ifdef NEED_READ_WRITE
 struct map *
-read_map(f, arg)
-     int (*f) ();
-     char *arg;
+read_map(int (*f) (void *), char *arg)
 {
     struct bitmap *map, *result, **mp;
     int page;
-    int bitno, lastno, thisno, prevno;
-    int i, j;
+    int bitno, lastno;
     int data;
 
 /* count, then startbitno, then bits. */
@@ -729,7 +700,7 @@ read_map(f, arg)
 	    continue;
 	page = NUMBERTOPAGE(bitno);
 	if (!map || map->m_page != page)
-	    for (mp = &result; map = *mp; mp = &map->m_next)
+	    for (mp = &result; (map = *mp); mp = &map->m_next)
 		if (map->m_page == page)
 		    break;
 	if (!map) {
@@ -742,7 +713,7 @@ read_map(f, arg)
 		    free_map((struct map *)result);
 		return 0;
 	    }
-	    bzero((char *)map->m_data, sizeof map->m_data);
+	    memset((char *) map->m_data, 0, sizeof map->m_data);
 	    map->m_page = page;
 	    map->m_next = 0;
 	    *mp = map;
@@ -752,10 +723,8 @@ read_map(f, arg)
     return (struct map *)result;
 }
 
-write_map(parm, f, arg)
-     struct map *parm;
-     int (*f) ();
-     char *arg;
+int 
+write_map(struct map *parm, int (*f) (void *, int), char *arg)
 {
     struct bitmap *map;
     int page;
