@@ -35,7 +35,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/sys/pioctl_nt.c,v 1.34.4.3 2006/08/28 17:18:36 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/sys/pioctl_nt.c,v 1.34.4.7 2007/07/18 13:03:44 jaltman Exp $");
 
 #include <afs/stds.h>
 #include <windows.h>
@@ -53,12 +53,13 @@ RCSID
 #include <osi.h>
 
 #include <cm.h>
-#include <cm_dir.h>
+#include <cm_server.h>
 #include <cm_cell.h>
 #include <cm_user.h>
 #include <cm_conn.h>
 #include <cm_scache.h>
 #include <cm_buf.h>
+#include <cm_dir.h>
 #include <cm_utils.h>
 #include <cm_ioctl.h>
 
@@ -90,6 +91,7 @@ CMtoUNIXerror(int cm_code)
     case CM_ERROR_NOACCESS:
 	return EACCES;
     case CM_ERROR_NOSUCHFILE:
+    case CM_ERROR_NOSUCHPATH:
 	return ENOENT;
     case CM_ERROR_INVAL:
 	return EINVAL;
@@ -115,6 +117,12 @@ CMtoUNIXerror(int cm_code)
 	return EDOM;		/* hack */
     case CM_ERROR_TOOMANYBUFS:
 	return EFBIG;		/* hack */
+    case CM_ERROR_ALLBUSY:
+        return EBUSY;
+    case CM_ERROR_ALLDOWN:
+        return ENOSYS;          /* hack */
+    case CM_ERROR_ALLOFFLINE:
+        return ENXIO;           /* hack */
     default:
 	if (cm_code > 0 && cm_code < EILSEQ)
 	    return cm_code;
@@ -991,6 +999,10 @@ pioctl(char *pathp, long opcode, struct ViceIoctl *blobp, int follow)
 
     MarshallString(&preq, fullPath);
     if (blobp->in_size) {
+        if (blobp->in_size > sizeof(preq.data) - (preq.mp - preq.data)*sizeof(char)) {
+            errno = E2BIG;
+            return -1;
+        }
 	memcpy(preq.mp, blobp->in, blobp->in_size);
 	preq.mp += blobp->in_size;
     }
